package org.artifactory.queue.model;

import lombok.EqualsAndHashCode;
import lombok.NonNull;
import lombok.Value;
import org.apache.commons.lang3.StringUtils;

/**
 * @author avishayh
 */
@Value
public class PersistentQueueRunningResult {

    private static final int MAX_MESSAGE_LENGTH = 4000;
    private static final PersistentQueueRunningResult SUCCESS = new PersistentQueueRunningResult(-1, null);

    long failedMarker;
    @EqualsAndHashCode.Exclude
    String errorMessage;

    private PersistentQueueRunningResult(long failedMarker, String errorMessage) {
        this.failedMarker = failedMarker;
        this.errorMessage = StringUtils.abbreviate(errorMessage, MAX_MESSAGE_LENGTH);
    }

    public static PersistentQueueRunningResult success() {
        return SUCCESS;
    }

    public static PersistentQueueRunningResult failure(long marker, @NonNull String errorMessage) {
        return new PersistentQueueRunningResult(marker, errorMessage);
    }

    public boolean isFailed() {
        return errorMessage != null;
    }
}
