package org.artifactory.queue.services;

import org.artifactory.queue.QueueStatus;
import org.artifactory.storage.db.event.entity.NodeEventCursorType;

/**
 * A "bridge" service interface between Events services and PersistentQueue framework
 * contains all the methods needed to handle the cursor.
 * @author avishayh
 */
public interface PersistentQueueCursorService {

    static String getCursorName(String repoKey, NodeEventCursorType type, String variant) {
        return repoKey + "-" + type.getName().toLowerCase() + variant + "-cursor";
    }

    void updateMarker(String cursorName, long newMarker);

    void resetMarker(String repoKey, String cursorName);

    void deleteMarker(String cursorName);

    /**
     * @param repoKey    - the repo key
     * @param cursorName - name of cursor
     * @param type       - type of cursor if need to create one
     * @return the current marker (eventId) for a cursor, if cursor doesn't exist creates one with initial value.
     */
    long getCurrentMarker(String repoKey, String cursorName, NodeEventCursorType type);

    QueueStatus getStatus(String repoKey, String cursorName);

    /**
     * Create cursor marker if not exists
     */
    void createMarkerIfNotExists(String repoKey, String cursorName, NodeEventCursorType type);
}
