package org.artifactory.storage.db.conversion.version.v221;

import org.artifactory.storage.db.version.ConversionPredicate;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;
import org.jfrog.storage.util.DbUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.SQLException;
import java.util.function.BiPredicate;


/**
 * @author andreik
 */
public class V221ConversionPredicateNodePropsIndex implements ConversionPredicate {

    private static final Logger log = LoggerFactory.getLogger(V221ConversionPredicateNodePropsIndex.class);
    private static final String PROP_KEY_COLUMN = "prop_key";
    private static final String NODE_PROPS_PROP_KEY_VALUE_INDEX = "node_props_prop_key_value_idx";

    @Override
    public BiPredicate<JdbcHelper, DbType> condition() {
        return (jdbcHelper, dbType) -> {
            try {
                boolean indexExists = DbUtils.indexExists(jdbcHelper, "node_props", PROP_KEY_COLUMN,
                        NODE_PROPS_PROP_KEY_VALUE_INDEX, dbType);
                log.debug("Index {} exists: {}", NODE_PROPS_PROP_KEY_VALUE_INDEX, indexExists);
                return !indexExists;
            } catch (SQLException e) {
                log.error("Cannot run conversion 'v221' - Failed to check if index exists: ", e);
            }
            return false;
        };
    }
}
