package org.artifactory.storage.db.conversion.version.v230;

import lombok.NonNull;
import org.artifactory.storage.db.conversion.DbConversionException;
import org.artifactory.storage.db.version.ConversionPredicate;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;
import org.jfrog.storage.util.DbUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.function.BiPredicate;

import static org.artifactory.storage.db.event.error.dao.NodeEventsErrorsDAO.TABLE_NODE_EVENTS_ERRORS;
import static org.jfrog.storage.util.DbUtils.withConnection;

/**
 * @author andreik
 */
public class V230ConversionPredicate implements ConversionPredicate {

    private static final Logger log = LoggerFactory.getLogger(V230ConversionPredicate.class);

    @NonNull
    @Override
    public BiPredicate<JdbcHelper, DbType> condition() {
        return (jdbcHelper, dbType) -> {
            if (DbType.MSSQL != dbType) {
                return false;
            }
            try {
                // run conversion if column not exist
                if (withConnection(jdbcHelper, conn -> DbUtils
                        .isColumnNullable(conn.getMetaData(), dbType, TABLE_NODE_EVENTS_ERRORS, "event_category"))) {
                    log.info("Fixing a failed v224 conversion.");
                    return true;
                }
            } catch (Exception e) {
                log.error("Cannot run conversion 'v230' - Failed to retrieve schema metadata");
                throw new DbConversionException(
                        "Cannot run conversion 'v230' - Failed to retrieve schema metadata: " + e.getMessage(), e);
            }
            log.debug("Not running the the v224 DB conversion fix.");
            return false;
        };
    }
}
