package org.artifactory.storage.db.conversion.version.v233;

import lombok.extern.slf4j.Slf4j;
import org.artifactory.storage.db.conversion.DbConversionException;
import org.artifactory.storage.db.version.ConversionPredicate;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;
import org.jfrog.storage.util.DbUtils;

import java.util.function.BiPredicate;

import static org.artifactory.storage.db.fs.dao.NodesDao.TABLE_NAME;

/**
 * @author Hezi Cohen
 */

@Slf4j
public class V233RemoveIndexPredicate implements ConversionPredicate {

    private static final String COLUMN_NAME = "repo_path_checksum";
    private static final String INDEX_NAME = "nodes_repo_path_checksum";

    @Override
    public BiPredicate<JdbcHelper, DbType> condition() {
        return (jdbcHelper, dbType) -> {
            if (DbType.MYSQL != dbType && DbType.MARIADB != dbType) {
                return false;
            }
            try {
                boolean indexExists = DbUtils
                        .indexExists(jdbcHelper, TABLE_NAME, COLUMN_NAME, INDEX_NAME, dbType);
                if (indexExists) { // remove UNIQUE index if EXISTS
                    log.info("Removing unique index '{}' in table '{}'.", INDEX_NAME, TABLE_NAME);
                    return true;
                }
            } catch (Exception e) {
                String msg = "Cannot run conversion v233 (remove unique index) - Failed to retrieve data from DB";
                log.error(msg);
                throw new DbConversionException(msg + ": " + e.getMessage(), e);
            }
            log.debug("Not running the the v233 (remove unique index) DB conversion fix.");
            return false;
        };
    }
}
