package org.artifactory.storage.db.upgrades.itest.version;


import org.artifactory.storage.db.init.CommonDataSourceFactory;
import org.artifactory.storage.db.upgrades.common.UpgradeBaseTest;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JFrogDataSource;
import org.jfrog.storage.JdbcHelper;
import org.springframework.test.context.ActiveProfiles;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Test for the unlogged table annotation of table distributed_locks
 * Test only on PostgreSql
 *
 * @author shaharz
 * @see org.artifactory.storage.db.itest.spring.DbTestConfigFactoryWithExternalLockingDB
 */

@Test
@ActiveProfiles({"with-external-locking-db"})
public class V225UpgradePostgreSqlDistributedLocksUnloggedTest extends UpgradeBaseTest {


    private static final String IS_UNLOGGED_SQL =
            "SELECT (CASE WHEN relpersistence = 'u' THEN TRUE ELSE FALSE END) is_unlogged " +
                    "FROM pg_class " +
                    "WHERE relname = 'distributed_locks'";


    /**
     * Verify that postgresql distribute_locks table was converted to unlogged
     *
     * @throws Exception
     */
    public void testPostgreSQLDistributedLockedUnlogged() throws Exception {
        if (dbProperties.getDbType().equals(DbType.POSTGRESQL)) {
            Assert.assertTrue(isUnlogged(jdbcHelper));
        }
    }

    /**
     * Verify that postgresql distribute_locks external table was converted to unlogged
     *
     * @throws Exception
     */
    public void testPostgreSQLExternalDistributedLockedUnlogged() throws Exception {
        if (dbProperties.getDbType().equals(DbType.POSTGRESQL) && dbProperties.getLockingDbSpecificType().isPresent()) {
            JFrogDataSource lockingDBDataSource =
                    CommonDataSourceFactory.createDifferentDatasourceForDbLock(dbProperties, true);
            JdbcHelper lockingDBJdbcHelper = new JdbcHelper(lockingDBDataSource);
            Assert.assertTrue(isUnlogged(lockingDBJdbcHelper));
        }
    }

    private boolean isUnlogged(JdbcHelper jdbcHelper) throws SQLException {
        try (ResultSet resultSet = jdbcHelper.executeSelect(IS_UNLOGGED_SQL)) {
            return resultSet.next() && resultSet.getBoolean(1);
        }
    }
}

