package org.artifactory.storage.db.upgrades.itest.version;

import org.artifactory.storage.db.upgrades.common.UpgradeBaseTest;
import org.artifactory.storage.db.version.ArtifactoryDBVersion;
import org.artifactory.storage.db.version.converter.DBConverter;
import org.jfrog.storage.DbType;
import org.testng.SkipException;
import org.testng.annotations.Test;

import java.io.IOException;
import java.sql.SQLException;

import static org.artifactory.storage.db.fs.dao.NodesDao.TABLE_NAME;
import static org.jfrog.storage.util.DbUtils.createIndexIfDoesntExist;
import static org.jfrog.storage.util.DbUtils.dropIndexIfExists;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;
import static org.jfrog.storage.util.DbUtils.indexExists;

/**
 * @author Hezi Cohen
 */

@Test
public class V233UpgradeTest extends UpgradeBaseTest {

    private static final String COLUMN_NAME = "repo_path_checksum";
    private static final String INDEX_NAME = "nodes_repo_path_checksum";

    public void testRepoPathChecksumWhenIndexDoesntExist() throws SQLException, IOException {
        assertDBType();
        resetUpgradeToV230();
        dropIndexIfExists(jdbcHelper, dbProperties.getDbType(), TABLE_NAME, COLUMN_NAME, INDEX_NAME);
        assertFalse(indexExists(jdbcHelper, TABLE_NAME, COLUMN_NAME, INDEX_NAME, dbProperties.getDbType()));
        runConvertersAndAssertSuccess();
    }

    public void testRepoPathChecksumWhenIndexExists() throws SQLException, IOException {
        assertDBType();
        resetUpgradeToV230();
        createIndexIfDoesntExist(jdbcHelper, dbProperties.getDbType(), TABLE_NAME, COLUMN_NAME, INDEX_NAME);
        assertTrue(indexExists(jdbcHelper, TABLE_NAME, COLUMN_NAME, INDEX_NAME, dbProperties.getDbType()));
        runConvertersAndAssertSuccess();
    }

    private void assertDBType() {
        if (dbProperties.getDbType() != DbType.MYSQL && dbProperties.getDbType() != DbType.MARIADB) {
            String message = "This test is for MySQL/MariaDB only, found " + dbProperties.getDbType()
                    + ". Skipping test.";
            logger.info(message);
            throw new SkipException(message);
        }
    }

    private void resetUpgradeToV230() throws SQLException, IOException {
        resetToVersion(ArtifactoryDBVersion.v230);
        assertTrue(tableExists(TABLE_NAME), "Missing table " + TABLE_NAME);
        assertTrue(columnExists(TABLE_NAME, COLUMN_NAME));
    }

    private void runConvertersAndAssertSuccess() throws SQLException {
        for (DBConverter converter : ArtifactoryDBVersion.v233.getConverters()) {
            converter.convert(jdbcHelper, dbProperties.getDbType());
        }
        assertTrue(tableExists(TABLE_NAME), "Missing table " + TABLE_NAME);
        assertTrue(columnExists(TABLE_NAME, COLUMN_NAME));
        assertTrue(indexExists(jdbcHelper, TABLE_NAME, COLUMN_NAME, INDEX_NAME, dbProperties.getDbType()));
    }
}
