package org.artifactory.ui.rest.service.admin.security.keys.keypairs;

import com.google.common.collect.Maps;
import org.artifactory.addon.common.keys.KeyPairService;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.descriptor.config.MutableCentralConfigDescriptor;
import org.artifactory.descriptor.repo.FederatedRepoDescriptor;
import org.artifactory.descriptor.repo.KeyPairDescriptor;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.ArtifactoryRestResponse;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.ui.rest.model.admin.security.keys.KeyPairIModel;
import org.testng.annotations.Test;

import java.util.List;
import java.util.Map;

import static org.mockito.ArgumentMatchers.anyCollection;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;

/**
 * @author Alexei Vainshtein
 */
public class GetAllKeyPairsServiceTest {

    @Test
    public void testGetListOfReposPerKeyPair() {
        CentralConfigService centralConfigService = mock(CentralConfigService.class);
        MutableCentralConfigDescriptor descriptor = mock(MutableCentralConfigDescriptor.class);
        Map<String, FederatedRepoDescriptor> reposMap = Maps.newLinkedHashMap();
        String repoName = "repo";
        FederatedRepoDescriptor federatedRepo = new FederatedRepoDescriptor();
        KeyPairDescriptor keyPair = new KeyPairDescriptor();
        keyPair.setKey("key");
        federatedRepo.setPrimaryKeyPairDescriptor(keyPair);
        reposMap.put(repoName, federatedRepo);
        when(descriptor.getFederatedRepositoriesMap()).thenReturn(reposMap);
        when(centralConfigService.getMutableDescriptor()).thenReturn(descriptor);
        GetAllKeyPairsService service = new GetAllKeyPairsService(centralConfigService, mock(KeyPairService.class));
        List<String> keys = service.getListOfReposPerKeyPair("key");
        assertEquals(keys.size(), reposMap.size());
    }

    @Test
    public void testExecute() {
        CentralConfigService centralConfigService = mock(CentralConfigService.class);
        MutableCentralConfigDescriptor descriptor = mock(MutableCentralConfigDescriptor.class);
        when(centralConfigService.getMutableDescriptor()).thenReturn(descriptor);
        KeyPairDescriptor keyPair = new KeyPairDescriptor();
        keyPair.setKey("key");
        keyPair.setAlias("alias");
        keyPair.setKeyType(KeyPairDescriptor.RSA_TYPE);
        keyPair.setVaultType(KeyPairDescriptor.VAULT);

        KeyPairService keyPairService = mock(KeyPairService.class);
        when(keyPairService.getKeyPairs(anyCollection())).thenReturn(List.of(keyPair));
        GetAllKeyPairsService service = new GetAllKeyPairsService(centralConfigService, keyPairService);

        ArtifactoryRestRequest request = mock(ArtifactoryRestRequest.class);
        RestResponse response = new ArtifactoryRestResponse();
        service.execute(request, response);

        assertNotNull(response.getiModelList());
        KeyPairIModel keyPairIModel = (KeyPairIModel) response.getiModelList().iterator().next();
        assertEquals(keyPairIModel.getPairName(), keyPair.getKey());
        assertEquals(keyPairIModel.getAlias(), keyPair.getAlias());
        assertEquals(keyPairIModel.getSource(), keyPair.getVaultType());
        assertEquals(keyPairIModel.getPairType(), keyPair.getKeyType());
    }
}